@@ -1,12 +1,13 @@
 # coding=utf-8
 #
-# This file is part of Hypothesis (https://github.com/DRMacIver/hypothesis)
+# This file is part of Hypothesis, which may be found at
+# https://github.com/HypothesisWorks/hypothesis-python
 #
 # Most of this work is copyright (C) 2013-2015 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
-# https://github.com/DRMacIver/hypothesis/blob/master/CONTRIBUTING.rst for a
-# full list of people who may hold copyright, and consult the git log if you
-# need to determine who owns an individual contribution.
+# CONTRIBUTING.rst for a full list of people who may hold copyright, and
+# consult the git log if you need to determine who owns an individual
+# contribution.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
@@ -23,7 +24,6 @@ import time
 import inspect
 import functools
 import traceback
-from random import getstate as getglobalrandomstate
 from random import Random
 from collections import namedtuple
 
@@ -88,8 +88,15 @@ def reify_and_execute(
     print_example=False,
     is_final=False,
 ):
+    from hypothesis.strategies import random_module
+
     def run(data):
+        from hypothesis.control import note
+
         with BuildContext(is_final=is_final):
+            seed = data.draw(random_module()).seed
+            if seed != 0:
+                note('random.seed(%d)' % (seed,))
             args, kwargs = data.draw(search_strategy)
 
             if print_example:
@@ -286,11 +293,11 @@ def given(*generator_arguments, **generator_kwargs):
                 if label in settings.suppress_health_check:
                     return
                 message += (
-                    '\nSee http://hypothesis.readthedocs.org/en/latest/health'
+                    '\nSee https://hypothesis.readthedocs.io/en/latest/health'
                     'checks.html for more information about this. '
                 )
                 message += (
-                    'If you want to disable just this health check, add %s'
+                    'If you want to disable just this health check, add %s '
                     'to the suppress_health_check settings for this test.'
                 ) % (label,)
                 raise FailedHealthCheck(message)
@@ -313,7 +320,6 @@ def given(*generator_arguments, **generator_kwargs):
                 return
 
             if perform_health_check:
-                initial_state = getglobalrandomstate()
                 health_check_random = Random(random.getrandbits(128))
                 # We "pre warm" the health check with one draw to give it some
                 # time to calculate any cached data. This prevents the case
@@ -363,6 +369,11 @@ def given(*generator_arguments, **generator_kwargs):
                     except InvalidArgument:
                         raise
                     except Exception:
+                        if (
+                            HealthCheck.exception_in_generation in
+                            settings.suppress_health_check
+                        ):
+                            raise
                         report(traceback.format_exc())
                         if test_runner is default_new_style_executor:
                             fail_health_check(
@@ -370,7 +381,7 @@ def given(*generator_arguments, **generator_kwargs):
                                 'generation in initial health check. '
                                 'This indicates a bug in the strategy. '
                                 'This could either be a Hypothesis bug or '
-                                "an error in a function yo've passed to "
+                                "an error in a function you've passed to "
                                 'it to construct your data.',
                                 HealthCheck.exception_in_generation,
                             )
@@ -423,27 +434,10 @@ def given(*generator_arguments, **generator_kwargs):
                     ) % (count, runtime, filtered_draws, overruns),
                         HealthCheck.too_slow,
                     )
-                if getglobalrandomstate() != initial_state:
-                    fail_health_check(
-                        'Data generation depends on global random module. '
-                        'This makes results impossible to replay, which '
-                        'prevents Hypothesis from working correctly. '
-                        'If you want to use methods from random, use '
-                        'randoms() from hypothesis.strategies to get an '
-                        'instance of Random you can use. Alternatively, you '
-                        'can use the random_module() strategy to explicitly '
-                        'seed the random module.', HealthCheck.random_module,
-                    )
             last_exception = [None]
             repr_for_last_exception = [None]
-            performed_random_check = [False]
 
             def evaluate_test_data(data):
-                if perform_health_check and not performed_random_check[0]:
-                    initial_state = getglobalrandomstate()
-                    performed_random_check[0] = True
-                else:
-                    initial_state = None
                 try:
                     result = test_runner(data, reify_and_execute(
                         search_strategy, test,
@@ -465,20 +459,6 @@ def given(*generator_arguments, **generator_kwargs):
                     last_exception[0] = traceback.format_exc()
                     verbose_report(last_exception[0])
                     data.mark_interesting()
-                finally:
-                    if (
-                        initial_state is not None and
-                        getglobalrandomstate() != initial_state
-                    ):
-                        fail_health_check(
-                            'Your test used the global random module. '
-                            'This is unlikely to work correctly. You should '
-                            'consider using the randoms() strategy from '
-                            'hypothesis.strategies instead. Alternatively, '
-                            'you can use the random_module() strategy to '
-                            'explicitly seed the random module.',
-                            HealthCheck.random_module,
-                        )
 
             from hypothesis.internal.conjecture.engine import TestRunner
 
