@@ -1,12 +1,13 @@
 # coding=utf-8
 #
-# This file is part of Hypothesis (https://github.com/DRMacIver/hypothesis)
+# This file is part of Hypothesis, which may be found at
+# https://github.com/HypothesisWorks/hypothesis-python
 #
 # Most of this work is copyright (C) 2013-2015 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
-# https://github.com/DRMacIver/hypothesis/blob/master/CONTRIBUTING.rst for a
-# full list of people who may hold copyright, and consult the git log if you
-# need to determine who owns an individual contribution.
+# CONTRIBUTING.rst for a full list of people who may hold copyright, and
+# consult the git log if you need to determine who owns an individual
+# contribution.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
@@ -38,13 +39,6 @@ class UnsatisfiedAssumption(HypothesisException):
 
     """
 
-    def __init__(self, *args, **kwargs):
-        if args or kwargs:
-            super(UnsatisfiedAssumption, self).__init__(*args, **kwargs)
-        else:
-            super(UnsatisfiedAssumption, self).__init__(
-                u'Unsatisfied assumption')
-
 
 class BadTemplateDraw(HypothesisException):
 
@@ -67,25 +61,21 @@ class NoSuchExample(HypothesisException):
 
     """
 
-    def __init__(self, condition_string, extra=u''):
+    def __init__(self, condition_string, extra=''):
         super(NoSuchExample, self).__init__(
-            u'No examples found of condition %s%s' % (
+            'No examples found of condition %s%s' % (
                 condition_string, extra)
         )
 
 
-class DefinitelyNoSuchExample(NoSuchExample):
+class DefinitelyNoSuchExample(NoSuchExample):  # pragma: no cover
+    """Hypothesis used to be able to detect exhaustive coverage of a search
+    space and no longer can.
 
-    """We have considered the entire example space available and there are no
-    examples in it."""
+    This exception remains for compatibility reasons for now but can
+    never actually be thrown.
 
-    def __init__(self, condition_string, n_examples):
-        super(DefinitelyNoSuchExample, self).__init__(
-            condition_string, u' (all %d considered)' % (
-                n_examples,
-            )
-        )
-        self.n_examples = n_examples
+    """
 
 
 class NoExamples(HypothesisException):
@@ -154,6 +144,11 @@ class InvalidArgument(HypothesisException, TypeError):
     some manner incorrect."""
 
 
+class InvalidState(HypothesisException):
+
+    """The system is not in a state where you were allowed to do that."""
+
+
 class InvalidDefinition(HypothesisException, TypeError):
 
     """Used to indicate that a class definition was not well put together and
@@ -175,4 +170,10 @@ class HypothesisDeprecationWarning(HypothesisException, DeprecationWarning):
     pass
 
 
-warnings.simplefilter(u'once', HypothesisDeprecationWarning)
+warnings.simplefilter('once', HypothesisDeprecationWarning)
+
+
+class Frozen(HypothesisException):
+
+    """Raised when a mutation method has been called on a TestData object after
+    freeze() has been called."""
